@@ -192,7 +192,7 @@ class GitClient(VcsClientBase):
                 return False
         return True
 
-    def update(self, refname=None, verbose=False):
+    def update(self, refname=None, verbose=False, force_fetch=False):
         """
         interprets refname as a local branch, remote branch, tagname,
         hash, etc.
@@ -201,12 +201,17 @@ class GitClient(VcsClientBase):
         already on it, and to fast-forward, unless not a tracking
         branch. Else go untracked on tag or whatever refname is. Does
         not leave if current commit would become dangling.
+
+        :return: True if already up-to-date with remote or after successful fast_foward
         """
         # try calling git fetch just once per call to update()
         need_to_fetch = True
         if not self.detect_presence():
             return False
 
+        if force_fetch:
+            self._do_fetch(True)
+
         # are we on any branch?
         current_branch = self.get_branch()
         if current_branch:
@@ -227,7 +232,7 @@ class GitClient(VcsClientBase):
         # if same_branch and branch_parent is None:
         #   already on branch, nothing to pull as non-tracking branch
         if same_branch and branch_parent is not None:
-            if not self._do_fast_forward(need_to_fetch, verbose=verbose):
+            if not self._do_fast_forward(need_to_fetch, branch_parent=branch_parent, verbose=verbose):
                 return False
             need_to_fetch = False
         elif not same_branch:
@@ -270,6 +275,7 @@ class GitClient(VcsClientBase):
                 new_branch_parent = self.get_branch_parent(fetch=need_to_fetch)
                 if new_branch_parent is not None:
                     if not self._do_fast_forward(fetch=need_to_fetch,
+                                                 branch_parent=new_branch_parent,
                                                  verbose=verbose):
                         return False
 
@@ -477,7 +483,7 @@ class GitClient(VcsClientBase):
         :param refname: a git refname
         :param version: an SHA IDs (if partial, caller is responsible
           for mismatch)
-        :returns: True if version can be found in rev-list
+        :returns: True if version is an ancestor commit from refname
         """
         # to avoid listing unnecessarily many rev-ids, we cut off all
         # those we are definitely not interested in
@@ -494,7 +500,7 @@ class GitClient(VcsClientBase):
             version is not None and
             version!=''):
 
-            cmd = 'git rev-list %s %s --parents'%(sanitized(refname), sanitized('^%s'%version))
+            cmd = 'git rev-list %s ^%s --parents'%(sanitized(refname), sanitized(version))
             _, output, _ = run_shell_command(cmd, shell=True, cwd=self._path)
             for line in output.splitlines():
                 # can have 1, 2 or 3 elements (commit, parent1, parent2)
@@ -543,62 +549,82 @@ class GitClient(VcsClientBase):
         if result:
             return False
         # Gzip the tar file
-        tar_file = open(basepath + '.tar', 'rb')
-        gzip_file = gzip.open(basepath + '.tar.gz', 'wb')
-        gzip_file.writelines(tar_file)
-        tar_file.close()
-        gzip_file.close()
+        with open(basepath + '.tar', 'rb') as tar_file:
+            gzip_file = gzip.open(basepath + '.tar.gz', 'wb')
+            try:
+                gzip_file.writelines(tar_file)
+            finally:
+                gzip_file.close()
         # Clean up
         os.remove(basepath + '.tar')
         return True
 
-    def _do_fetch(self):
+    def _do_fetch(self, with_tags=False):
         """calls git fetch"""
-        value, _, _ = run_shell_command("git fetch",
+        cmd = "git fetch"
+        if with_tags:
+            cmd += " --tags"
+        value, _, _ = run_shell_command(cmd,
                                         cwd=self._path,
                                         shell=True,
                                         show_stdout=True)
         return value == 0
 
-    def _do_fast_forward(self, fetch=True, verbose=False):
+    def _do_fast_forward(self, fetch=True, branch_parent=None, verbose=False):
         """Execute git fetch if necessary, and if we can fast-foward,
-        do so to the last fetched version using git rebase. Returns
-        False on command line failures"""
-        parent = self.get_branch_parent(fetch=fetch)
-        if parent is not None and self.rev_list_contains("remotes/origin/%s" % parent,
-                                                         self.get_version(),
-                                                         fetch=False):
-            if verbose:
-                print("Rebasing repository")
-            # Rebase, do not pull, because somebody could have
-            # commited in the meantime.
-            if LooseVersion(self.gitversion) >= LooseVersion('1.7.1'):
-                # --keep allows o rebase even with local changes, as long as
-                # local changes are not in files that change between versions
-                cmd = "git reset --keep remotes/origin/%s"%parent
-                value, _, _ = run_shell_command(cmd,
-                                                shell=True,
-                                                cwd=self._path,
-                                                show_stdout=True,
-                                                verbose=verbose)
-                if value == 0:
-                    return True
-            else:
-                verboseflag = ''
-                if verbose:
-                    verboseflag = '-v'
-                # prior to version 1.7.1, git does not know --keep
-                # Do not merge, rebase does nothing when there are local changes
-                cmd = "git rebase %s remotes/origin/%s"%(verboseflag, parent)
-                value, _, _ = run_shell_command(cmd,
-                                                shell=True,
-                                                cwd=self._path,
-                                                show_stdout=True,
-                                                verbose=verbose)
-                if value == 0:
-                    return True
+        do so to the last fetched version using git rebase.
+
+        :param branch_parent: name of branch we track
+        :param fetch: whether fetch should be done first for remote refs
+        :returns: True if up-to-date or after succesful fast-forward
+        """
+        assert branch_parent is not None
+        current_version = self.get_version()
+        parent_version = self.get_version("remotes/origin/%s" % branch_parent)
+        if current_version == parent_version:
+            return True
+        # check if we are true ancestor of tracked branch
+        if not self.rev_list_contains(parent_version,
+                                      current_version,
+                                      fetch=fetch):
+            # if not rev_list_contains this version, we are on same
+            # commit (checked before), have advanced, or have diverged.
+            # Now check whether tracked branch is a true ancestor of us
+            if self.rev_list_contains(current_version,
+                                      parent_version,
+                                      fetch=False):
+                return True
             return False
-        return True
+        if verbose:
+            print("Rebasing repository")
+        # Rebase, do not pull, because somebody could have
+        # commited in the meantime.
+        if LooseVersion(self.gitversion) >= LooseVersion('1.7.1'):
+            # --keep allows o rebase even with local changes, as long as
+            # local changes are not in files that change between versions
+            cmd = "git reset --keep remotes/origin/%s" % branch_parent
+            value, _, _ = run_shell_command(cmd,
+                                            shell=True,
+                                            cwd=self._path,
+                                            show_stdout=True,
+                                            verbose=verbose)
+            if value == 0:
+                return True
+        else:
+            verboseflag = ''
+            if verbose:
+                verboseflag = '-v'
+            # prior to version 1.7.1, git does not know --keep
+            # Do not merge, rebase does nothing when there are local changes
+            cmd = "git rebase %s remotes/origin/%s" % (verboseflag, branch_parent)
+            value, _, _ = run_shell_command(cmd,
+                                            shell=True,
+                                            cwd=self._path,
+                                            show_stdout=True,
+                                            verbose=verbose)
+            if value == 0:
+                return True
+        return False
 
     def _do_checkout(self, refname, fetch=True, verbose=False):
         """meaning git checkout, not vcstools checkout. This works
